"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const minimatch = require("minimatch");
const linterConfigHelpers_1 = require("./linterConfigHelpers");
const CompilerHost_1 = require("./CompilerHost");
const FsHelper_1 = require("./FsHelper");
const issue_1 = require("./issue");
class ApiIncrementalChecker {
    constructor({ typescript, context, programConfigFile, compilerOptions, linterConfigFile, linterAutoFix, eslinter, vue, checkSyntacticErrors = false, resolveModuleName, resolveTypeReferenceDirective }) {
        this.linterConfigs = {};
        this.linterExclusions = [];
        this.currentLintErrors = new Map();
        this.currentEsLintErrors = new Map();
        this.lastUpdatedFiles = [];
        this.lastRemovedFiles = [];
        this.getLinterConfig = linterConfigHelpers_1.makeGetLinterConfig(this.linterConfigs, this.linterExclusions, this.context);
        this.context = context;
        this.linterConfigFile = linterConfigFile;
        this.linterAutoFix = linterAutoFix;
        this.eslinter = eslinter;
        this.hasFixedConfig = typeof this.linterConfigFile === 'string';
        this.initLinterConfig();
        this.tsIncrementalCompiler = new CompilerHost_1.CompilerHost(typescript, vue, programConfigFile, compilerOptions, checkSyntacticErrors, resolveModuleName, resolveTypeReferenceDirective);
    }
    initLinterConfig() {
        if (!this.linterConfig && this.hasFixedConfig) {
            this.linterConfig = linterConfigHelpers_1.loadLinterConfig(this.linterConfigFile);
            if (this.linterConfig.linterOptions &&
                this.linterConfig.linterOptions.exclude) {
                // Pre-build minimatch patterns to avoid additional overhead later on.
                // Note: Resolving the path is required to properly match against the full file paths,
                // and also deals with potential cross-platform problems regarding path separators.
                this.linterExclusions = this.linterConfig.linterOptions.exclude.map(pattern => new minimatch.Minimatch(path.resolve(pattern)));
            }
        }
    }
    createLinter(program) {
        // tslint:disable-next-line:no-implicit-dependencies
        const { Linter } = require('tslint');
        return new Linter({ fix: this.linterAutoFix }, program);
    }
    hasTsLinter() {
        return !!this.linterConfigFile;
    }
    hasEsLinter() {
        return this.eslinter !== undefined;
    }
    isFileExcluded(filePath) {
        return (filePath.endsWith('.d.ts') ||
            this.linterExclusions.some(matcher => matcher.match(filePath)));
    }
    nextIteration() {
        // do nothing
    }
    getTypeScriptIssues(_cancellationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const tsDiagnostics = yield this.tsIncrementalCompiler.processChanges();
            this.lastUpdatedFiles = tsDiagnostics.updatedFiles;
            this.lastRemovedFiles = tsDiagnostics.removedFiles;
            return issue_1.createIssuesFromTsDiagnostics(tsDiagnostics.results);
        });
    }
    getTsLintIssues(_cancellationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const updatedFile of this.lastUpdatedFiles) {
                if (this.isFileExcluded(updatedFile)) {
                    continue;
                }
                try {
                    const linter = this.createLinter(this.tsIncrementalCompiler.getProgram());
                    const config = this.hasFixedConfig
                        ? this.linterConfig
                        : this.getLinterConfig(updatedFile);
                    if (!config) {
                        continue;
                    }
                    // const source = fs.readFileSync(updatedFile, 'utf-8');
                    linter.lint(updatedFile, undefined, config);
                    const lints = linter.getResult();
                    this.currentLintErrors.set(updatedFile, lints);
                }
                catch (e) {
                    if (FsHelper_1.fileExistsSync(updatedFile) &&
                        // check the error type due to file system lag
                        !(e instanceof Error) &&
                        !(e.constructor.name === 'FatalError') &&
                        !(e.message && e.message.trim().startsWith('Invalid source file'))) {
                        // it's not because file doesn't exist - throw error
                        throw e;
                    }
                }
                for (const removedFile of this.lastRemovedFiles) {
                    this.currentLintErrors.delete(removedFile);
                }
            }
            const allLints = [];
            for (const [, value] of this.currentLintErrors) {
                allLints.push(...value.failures);
            }
            return issue_1.createIssuesFromTsLintRuleFailures(allLints);
        });
    }
    getEsLintIssues(cancellationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const removedFile of this.lastRemovedFiles) {
                this.currentEsLintErrors.delete(removedFile);
            }
            for (const updatedFile of this.lastUpdatedFiles) {
                cancellationToken.throwIfCancellationRequested();
                if (this.isFileExcluded(updatedFile)) {
                    continue;
                }
                const report = this.eslinter.getReport(updatedFile);
                if (report !== undefined) {
                    this.currentEsLintErrors.set(updatedFile, report);
                }
                else if (this.currentEsLintErrors.has(updatedFile)) {
                    this.currentEsLintErrors.delete(updatedFile);
                }
            }
            const reports = Array.from(this.currentEsLintErrors.values());
            return issue_1.createIssuesFromEsLintReports(reports);
        });
    }
}
exports.ApiIncrementalChecker = ApiIncrementalChecker;
//# sourceMappingURL=ApiIncrementalChecker.js.map